@@ -6,10 +6,6 @@
 
 #include <algorithm>
 
-#if defined(__linux__) || defined(__APPLE__) || defined(__FreeBSD__)
-#include <dlfcn.h>
-#endif
-
 #define GPA_IMPL(x)                                                                                                    \
     if (strcmp(pName, #x) == 0) {                                                                                      \
         return reinterpret_cast<PFN_vkVoidFunction>(shim_##x);                                                         \
@@ -143,11 +139,11 @@ VKAPI_ATTR void VKAPI_CALL shim_vkGetPhysicalDeviceFeatures2KHR(VkPhysicalDevice
     VkBaseOutStructure* current = static_cast<VkBaseOutStructure*>(pFeatures->pNext);
     while (current) {
         for (const auto& features_pNext : phys_dev.features_pNextChain) {
-            if (features_pNext.sType == current->sType) {
+            VkBaseOutStructure structure_data{};
+            memcpy(&structure_data, features_pNext.data(), sizeof(structure_data));
+            if (structure_data.sType == current->sType) {
                 VkBaseOutStructure* next = static_cast<VkBaseOutStructure*>(current->pNext);
-                std::memcpy(static_cast<void*>(current),
-                    static_cast<const void*>(&features_pNext),
-                    get_pnext_chain_struct_size(features_pNext.sType));
+                std::memcpy(static_cast<void*>(current), features_pNext.data(), features_pNext.size());
                 // Repair pNext void* since we clobbered it in the memcpy
                 current->pNext = next;
                 break;
@@ -189,7 +185,7 @@ VKAPI_ATTR VkResult VKAPI_CALL shim_vkCreateDevice(VkPhysicalDevice physicalDevi
     if (pCreateInfo->pEnabledFeatures) {
         new_feats = *pCreateInfo->pEnabledFeatures;
     }
-    std::vector<vkb::detail::GenericFeaturesPNextNode> new_chain;
+    std::vector<std::vector<char>> new_chain;
     std::vector<const char*> created_extensions;
     for (uint32_t i = 0; i < pCreateInfo->enabledExtensionCount; i++) {
         created_extensions.push_back(pCreateInfo->ppEnabledExtensionNames[i]);
@@ -199,9 +195,10 @@ VKAPI_ATTR VkResult VKAPI_CALL shim_vkCreateDevice(VkPhysicalDevice physicalDevi
         const auto* chain = static_cast<const VkBaseOutStructure*>(pNext_chain);
         const void* next = chain->pNext;
         if (check_if_features2_struct(chain->sType) > 0) {
-            vkb::detail::GenericFeaturesPNextNode node;
-            std::memcpy(static_cast<void*>(&node), pNext_chain, get_pnext_chain_struct_size(chain->sType));
-            new_chain.push_back(node);
+            std::vector<char> new_node;
+            new_node.resize(get_pnext_chain_struct_size(chain->sType));
+            std::memcpy(new_node.data(), pNext_chain, new_node.size());
+            new_chain.push_back(new_node);
         }
         if (chain->sType == VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FEATURES_2) {
             new_feats = static_cast<const VkPhysicalDeviceFeatures2*>(pNext_chain)->features;
@@ -402,49 +399,17 @@ PFN_vkVoidFunction shim_vkGetInstanceProcAddr([[maybe_unused]] VkInstance instan
     return nullptr;
 }
 
+#if defined(__GNUC__) && __GNUC__ >= 4
+#define EXPORT_FUNCTION __attribute__((visibility("default")))
+#elif defined(WIN32)
+#define EXPORT_FUNCTION __declspec(dllexport)
+#else
+#define EXPORT_FUNCTION
+#endif
+
+
 extern "C" {
-VKAPI_ATTR PFN_vkVoidFunction VKAPI_CALL vkGetInstanceProcAddr(VkInstance instance, const char* pName) {
+EXPORT_FUNCTION VKAPI_ATTR PFN_vkVoidFunction VKAPI_CALL vkGetInstanceProcAddr(VkInstance instance, const char* pName) {
     return shim_vkGetInstanceProcAddr(instance, pName);
 }
-
-#if defined(__linux__) || defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__GNU__)
-#define DLSYM_FUNC_NAME dlsym
-
-#elif defined(__APPLE__)
-#define DLSYM_FUNC_NAME my_dlsym
-#endif
-
-using PFN_DLSYM = void* (*)(void* handle, const char* symbol);
-
-
-#if defined(__APPLE__)
-#define real_dlsym dlsym
-#else
-PFN_DLSYM real_dlsym = nullptr;
-#endif
-
-void* DLSYM_FUNC_NAME([[maybe_unused]] void* handle, const char* symbol) {
-    if (strcmp(symbol, "vkGetInstanceProcAddr") == 0) {
-        return reinterpret_cast<void*>(shim_vkGetInstanceProcAddr);
-    }
-
-    return nullptr;
-}
-
-/* Shiming functions on apple is limited by the linker prefering to not use functions in the
- * executable in loaded dylibs. By adding an interposer, we redirect the linker to use our
- * version of the function over the real one, thus shimming the system function.
- */
-#if defined(__APPLE__)
-#define MACOS_ATTRIB __attribute__((section("__DATA,__interpose")))
-#define VOIDCP_CAST(_func) reinterpret_cast<const void*>(&_func)
-
-struct Interposer {
-    const void* shim_function;
-    const void* underlying_function;
-};
-
-__attribute__((used)) static Interposer _interpose_dlsym MACOS_ATTRIB = { VOIDCP_CAST(my_dlsym), VOIDCP_CAST(dlsym) };
-
-#endif
 }
